//Das Programm hat noch keinen Namen und ist eine IDE fr FreeBASIC
//Copyright (C) 2009  Thomas Stber
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA





#include <QtGui>
#include "highlighter.h"


extern QFont fontEdit;

Highlighter::Highlighter(QTextDocument *parent,QTextEdit *e)
        : QSyntaxHighlighter(parent)
{
    QSettings setting("syntax.ini",QSettings::IniFormat);
    edit = e;
    HighlightingRule rule;



    //Normaler Text
    stringFormat.setFont(fontEdit);
    stringFormat.setForeground(Qt::black);
    rule.ex = QRegExp(".");
    rule.format = stringFormat;
    highlightingVector.append(rule);



    //Schlsselwrter
    keywordFormat.setFont(fontEdit);
    QString color = setting.value("Syntax/keywordscolor").toString();
    QStringList color_list = color.split(",");
    keywordFormat.setForeground(QBrush(QColor(color_list[0].toInt(),color_list[1].toInt(),color_list[2].toInt())));
    QStringList keywordPatterns;
    QString s = setting.value("Syntax/keywords").toString();
    s.replace("\n","");
    s.replace(" ","");
    keywordPatterns = s.split("<<");
    for (int i = 0;i < keywordPatterns.count();i++)
    {
        keywordPatterns[i] = QString("\\b").append(keywordPatterns[i]).append("\\b");
    }
    foreach (const QString &pattern, keywordPatterns) {
        rule.ex = QRegExp(pattern);
        rule.format = keywordFormat;
        highlightingVector.append(rule);
    }

    //Datentypen
    datatypeFormat.setFont(fontEdit);
    color = setting.value("Syntax/datatypescolor").toString();
    color_list = color.split(",");
    datatypeFormat.setForeground(QBrush(QColor(color_list[0].toInt(),color_list[1].toInt(),color_list[2].toInt())));
    QStringList datatypePatterns;
    s = setting.value("Syntax/datatypes").toString();
    s.replace("\n","");
    s.replace(" ","");
    datatypePatterns = s.split("<<");
    for (int i = 0;i < datatypePatterns.count();i++)
    {
        datatypePatterns[i] = QString("\\b").append(datatypePatterns[i]).append("\\b");
    }
    foreach (const QString &pattern, datatypePatterns) {
        rule.ex = QRegExp(pattern);
        rule.format = datatypeFormat;
        highlightingVector.append(rule);
    }


    //Strings
    stringFormat.setFont(fontEdit);
    color = setting.value("Syntax/stringcolor").toString();
    color_list = color.split(",");
    stringFormat.setForeground(QBrush(QColor(color_list[0].toInt(),color_list[1].toInt(),color_list[2].toInt())));




    //Kommentar
    commentFormat.setFont(fontEdit);
    commentFormat.setForeground(Qt::gray);
    rule.ex = QRegExp("'[^\n]*");
    rule.format = commentFormat;
    highlightingVector.append(rule);



    //Zahlen
    numberFormat.setFont(fontEdit);
    color = setting.value("Syntax/numbercolor").toString();
    color_list = color.split(",");
    numberFormat.setForeground(QBrush(QColor(color_list[0].toInt(),color_list[1].toInt(),color_list[2].toInt())));
    rule.ex = QRegExp("\\b[0-9]+\\b");
    rule.format = numberFormat;
    highlightingVector.append(rule);
}














void Highlighter::highlightBlock(const QString &text)
{
    //Jede Regel einmal anwenden
    int length;
    int index;
    for (int i = 0;i < highlightingVector.count();i++)
    {
        QRegExp ex(highlightingVector[i].ex);
        ex.setCaseSensitivity(Qt::CaseInsensitive);  //FreeBASIC ist CaseInsensitive was hier festgelegt wird
        index = ex.indexIn(text);
        for (;index >= 0;)  //Solange bis Index -1 ist also nichts gefunden wurde
        {
            length = ex.matchedLength();
            setFormat(index, length, highlightingVector[i].format);  //Das eigentlich hervorheben
            index = ex.indexIn(text, index + length);                //Beim nchsten mal erst hinter dieser Stelle Anfangen
        }
    }
    setCurrentBlockState(0);    //Block ist OK
    highlightStrings(text);     //hebt noch zustzlich Strings hervor


}





//Rekursive Funktion. Notwendig um Syntaxfehler bei Strings zu beheben.
int Highlighter::findEnd(const QString &text, int start)
{
    QRegExp ex("\"");
    int end = ex.indexIn(text,start);
    if (end == -1)
    {
    }
    QRegExp ex2("\"\"");
    int end2 = ex2.indexIn(text,start);
    if (end2 != -1)
    {
        end = end2;
    }
    if (text.mid(end-1,0) != "\\") return end;
    return findEnd(text,end+1);
}




//hebt Strings hervor.
/*void Highlighter::highlightStrings(const QString &text)
{
    QRegExp ex("\"");
    QRegExp ex2("\"\"");
    int start = 0;
    int start2;
    int end;
    int end2;
    for(;;)
    {
        start = ex.indexIn(text,start);
        start2 = start;
        if (start != -1)
        {
            for (;;)
            {
                end = ex.indexIn(text,start2+1);
                end2 = ex2.indexIn(text,start2+1);

                if (end2 != -1)
                {
                    end = end2+1;
                    break;
                }
                if (text.mid(end-1,1) == "\\")
                {
                    start2 = end+1;
                    continue;
                }
                break;
            }
            if (end != -1)
            {
                setFormat(start,end-start+1, stringFormat);
                start = start + (end-start+1);
            }  else
            {
                break;
            }
        }
        if (start == -1)
        {
            break;
        }
    }
}
*/








void Highlighter::highlightStrings(const QString &text)
{
    QRegExp ex("\"");
    int start = 0;
    int start2;
    int end;
    int end2;
    for(;;)
    {
        start = ex.indexIn(text,start);
        if (start != -1)
        {
            end = ex.indexIn(text,start+1);
            if (end != -1)
            {
                setFormat(start,end-start+1, stringFormat);
                start = start + (end-start+1);
            }  else
            {
                break;
            }
        }
        if (start == -1)
        {
            break;
        }
    }
}
